#include "started_pch.h"
// For standard C++ printing (std::cout)
#include <iostream>
// For Multicast Group corbaloc reading
#include <fstream>
#include <string>
// For Naming Service capabilities
#include <orbsvcs/CosNamingC.h>
// For Multicast capabilities
#include <orbsvcs/PortableGroup/MIOP.h>
#include <orbsvcs/PortableGroup/GOA.h>
// The Servant
#include "SimpleInterface_i.h"
int main (int argc, char *argv[])
{
    // Use ACE-specific TRY/CATCH mechanism
    ACE_TRY_NEW_ENV
    {
        // Initialize and error check ORB
        CORBA::ORB_var orb = CORBA::ORB_init (argc, argv);
        if (CORBA::is_nil(orb.in()))
        {
            std::cout << "Could not initialize ORB" << std::endl;
            exit(1);
        }
        // Initialize and error check POA objects (Root POA, POA Manager)
        CORBA::Object_var poa_object = orb->resolve_initial_references("RootPOA");
        if (CORBA::is_nil(poa_object.in()))
        {
            std::cout << "Could not initialize POA Object" << std::endl;
            exit(1);
        }
        PortableGroup::GOA_var root_poa = PortableGroup::GOA::_narrow
                                          (poa_object.in ());
        if (CORBA::is_nil(root_poa.in()))
        {
            std::cout << "Could not initialize Root POA" << std::endl;
        }
        PortableServer::POAManager_var poa_manager = root_poa->the_POAManager ();
        if (CORBA::is_nil(poa_manager.in()))
        {
            std::cout << "Could not initialize POA Manager" << std::endl;
            exit(1);
        }
        // Create new POA that can have multiple IDs for each Servant
        PortableServer::IdUniquenessPolicy_var uniqueness =
                root_poa->create_id_uniqueness_policy(PortableServer::MULTIPLE_ID);
        CORBA::PolicyList policy_list;
        policy_list.length(1);
        policy_list[0] =
                PortableServer::IdUniquenessPolicy::_duplicate(uniqueness);
        PortableServer::POA_var poa = root_poa->create_POA(
                "child",
                poa_manager,
                policy_list);
        if (CORBA::is_nil(poa.in()))
        {
            std::cout << "Could not initialize POA" << std::endl;
            exit(1);
        }
        PortableGroup::GOA_var goa = PortableGroup::GOA::_narrow(poa.in());
        if (CORBA::is_nil(goa.in()))
        {
            std::cout << "Could not initialize GOA" << std::endl;
            exit(1);
        }
        // Initialize and error check Naming Service objects
        CORBA::Object_var NameServiceObjectVar = orb->resolve_initial_references("NameService");
        if (CORBA::is_nil(NameServiceObjectVar.in()))
        {
            std::cout << "Could not initialize Naming Service" << std::endl;
            exit(1);
        }
        CosNaming::NamingContext_var NamingContextVar =
                CosNaming::NamingContext::_narrow(NameServiceObjectVar.in());
        if (CORBA::is_nil(NamingContextVar.in()))
        {
            std::cout << "Could not initialize Naming Context" << std::endl;
            exit(1);
        }
        // Obtain multiple Group corbalocs from file
        std::ifstream corbaLocFile1("group1.ior");
        std::string groupCorbaloc1;
        corbaLocFile1 >> groupCorbaloc1;
        std::ifstream corbaLocFile2("group2.ior");
        std::string groupCorbaloc2;
        corbaLocFile2 >> groupCorbaloc2;
        // Convert Group corbaloc into Object_var, then ObjectId_var
        CORBA::Object_var McastTemp1 = orb->string_to_object(groupCorbaloc1.c_str());
        PortableServer::ObjectId_var id1 = goa->create_id_for_reference(McastTemp1.in());
        CORBA::Object_var McastTemp2 = orb->string_to_object(groupCorbaloc2.c_str());
        PortableServer::ObjectId_var id2 = goa->create_id_for_reference(McastTemp2.in());
        // Initialize and activate Servant object
        SimpleInterface_i servant_impl (orb.in ());
        goa->activate_object_with_id(id1.in(), &servant_impl);
        goa->activate_object_with_id(id2.in(), &servant_impl);
        // Place Servants name in the Naming Service
        // (Multicast and Unicast)
        CosNaming::Name MCastName1(1);
        MCastName1.length(1);
        MCastName1[0].id = CORBA::string_dup("MCAST1");
        NamingContextVar->rebind(MCastName1, McastTemp1.in());
        CosNaming::Name MCastName2(1);
        MCastName2.length(1);
        MCastName2[0].id = CORBA::string_dup("MCAST2");
        NamingContextVar->rebind(MCastName2, McastTemp2.in());
        // Activate POA, run ORB (Servant now listening for incoming requests)
poa_manager->activate ();
        orb->run();
        // POA and ORB cleanup
        root_poa->destroy(1, 1);
        orb->destroy();
    }
    ACE_CATCHANY
    {
        ACE_PRINT_EXCEPTION (ACE_ANY_EXCEPTION,
                             "Exception caught:");
        return 1;
    }
    ACE_ENDTRY;
    return 0;
}
